@@ -1,9 +1,11 @@
 import asyncio
 import importlib
+import inspect
 import json
 import logging
 import os
 import warnings
+from contextvars import ContextVar
 from copy import deepcopy
 from inspect import isclass
 from typing import Any, Coroutine, Dict, List, Optional, Tuple, Type, Union, cast
@@ -26,11 +28,6 @@ from tortoise.queryset import QuerySet
 from tortoise.transactions import current_transaction_map
 from tortoise.utils import generate_schema_for_client
 
-try:
-    from contextvars import ContextVar
-except ImportError:  # pragma: nocoverage
-    from aiocontextvars import ContextVar  # type: ignore
-
 logger = logging.getLogger("tortoise")
 
 
@@ -74,6 +71,7 @@ class Tortoise:
                     "table":                str     # DB table name
                     "abstract":             bool    # Is the model Abstract?
                     "description":          str     # Description of table (nullable)
+                    "docstring":            str     # Model docstring (nullable)
                     "unique_together":      [...]   # List of List containing field names that
                                                     #  are unique together
                     "pk_field":             {...}   # Primary key field
@@ -105,6 +103,7 @@ class Tortoise:
                     "indexed":      bool    # Is the field indexed?
                     "default":      ...     # The default value (coerced to int/float/str/bool/null)
                     "description":  str     # Description of the field (nullable)
+                    "docstring":    str     # Field docstring (nullable)
                 }
 
             When ``serializable=False`` is specified some fields are not coerced to valid
@@ -122,6 +121,8 @@ class Tortoise:
         def _type_name(typ: Type) -> str:
             if typ.__module__ == "builtins":
                 return typ.__name__
+            if typ.__module__ == "typing":
+                return str(typ).replace("typing.", "")
             return f"{typ.__module__}.{typ.__name__}"
 
         def model_name(typ: Type[Model]) -> str:
@@ -141,7 +142,10 @@ class Tortoise:
             try:
                 return _type_name(typ)
             except AttributeError:
-                return [_type_name(_typ) for _typ in typ]
+                try:
+                    return [_type_name(_typ) for _typ in typ]  # pragma: nobranch
+                except TypeError:
+                    return str(typ)
 
         def default_name(default: Any) -> Optional[Union[int, float, str, bool]]:
             if isinstance(default, (int, float, str, bool, type(None))):
@@ -167,6 +171,7 @@ class Tortoise:
                 "indexed": field.index or field.unique,
                 "default": default_name(field.default) if serializable else field.default,
                 "description": field.description,
+                "docstring": field.docstring,
             }
 
             # Delete db fields for non-db fields
@@ -194,6 +199,7 @@ class Tortoise:
             "table": model._meta.table,
             "abstract": model._meta.abstract,
             "description": model._meta.table_description or None,
+            "docstring": inspect.cleandoc(model.__doc__ or "") or None,
             "unique_together": model._meta.unique_together or [],
             "pk_field": describe_field(model._meta.pk_attr),
             "data_fields": [
@@ -272,6 +278,8 @@ class Tortoise:
             """
             Test, if app and model really exist. Throws a ConfigurationError with a hopefully
             helpful message. If successfull, returns the requested model.
+
+            :raises ConfigurationError: If no such app exists.
             """
             try:
                 return cls.apps[related_app_name][related_model_name]
@@ -288,6 +296,8 @@ class Tortoise:
             Test, if reference follow the official naming conventions. Throws a
             ConfigurationError with a hopefully helpful message. If successfull,
             returns the app and the model name.
+
+            :raises ConfigurationError: If no model reference is invalid.
             """
             items = reference.split(".")
             if len(items) != 2:  # pragma: nocoverage
@@ -349,13 +359,12 @@ class Tortoise:
                     key_fk_object.description = fk_object.description
                     if fk_object.source_field:
                         key_fk_object.source_field = fk_object.source_field
-                        fk_object.source_field = key_field
                     else:
-                        fk_object.source_field = key_field
                         key_fk_object.source_field = key_field
                     model._meta.add_field(key_field, key_fk_object)
 
                     fk_object.model_class = related_model
+                    fk_object.source_field = key_field
                     backward_relation_name = fk_object.related_name
                     if backward_relation_name is not False:
                         if not backward_relation_name:
@@ -412,13 +421,12 @@ class Tortoise:
                     key_o2o_object.description = o2o_object.description
                     if o2o_object.source_field:
                         key_o2o_object.source_field = o2o_object.source_field
-                        o2o_object.source_field = key_field
                     else:
-                        o2o_object.source_field = key_field
                         key_o2o_object.source_field = key_field
                     model._meta.add_field(key_field, key_o2o_object)
 
                     o2o_object.model_class = related_model
+                    o2o_object.source_field = key_field
                     backward_relation_name = o2o_object.related_name
                     if backward_relation_name is not False:
                         if not backward_relation_name:
@@ -543,6 +551,30 @@ class Tortoise:
             cls._connections[name] = connection
             current_transaction_map[name] = ContextVar(name, default=connection)
 
+    @classmethod
+    def init_models(
+        cls, models_paths: List[str], app_label: str, _init_relations: bool = True
+    ) -> None:
+        """
+        Early initialisation of Tortoise ORM Models.
+
+        Initialise the relationships between Models.
+        This does not initialise any database connection.
+
+        :param models_paths: A list of model paths to initialise
+        :param app_label: The app label, e.g. 'models'
+
+        :raises ConfigurationError: If models are invalid.
+        """
+        app_models: List[Type[Model]] = []
+        for module in models_paths:
+            app_models += cls._discover_models(module, app_label)
+
+        cls.apps[app_label] = {model.__name__: model for model in app_models}
+
+        if _init_relations:
+            cls._init_relations()
+
     @classmethod
     def _init_apps(cls, apps_config: dict) -> None:
         for name, info in apps_config.items():
@@ -554,16 +586,11 @@ class Tortoise:
                         info.get("default_connection", "default"), name
                     )
                 )
-            app_models: List[Type[Model]] = []
-            for module in info["models"]:
-                app_models += cls._discover_models(module, name)
 
-            models_map = {}
-            for model in app_models:
+            cls.init_models(info["models"], name, _init_relations=False)
+
+            for model in cls.apps[name].values():
                 model._meta.default_connection = info.get("default_connection", "default")
-                models_map[model.__name__] = model
-
-            cls.apps[name] = models_map
 
         cls._init_relations()
 
@@ -612,57 +639,51 @@ class Tortoise:
         You can configure using only one of ``config``, ``config_file``
         and ``(db_url, modules)``.
 
-        Parameters
-        ----------
-        config:
+        :param config:
             Dict containing config:
 
-            Example
-            -------
+            .. admonition:: Example
 
-            .. code-block:: python3
+                .. code-block:: python3
 
-                {
-                    'connections': {
-                        # Dict format for connection
-                        'default': {
-                            'engine': 'tortoise.backends.asyncpg',
-                            'credentials': {
-                                'host': 'localhost',
-                                'port': '5432',
-                                'user': 'tortoise',
-                                'password': 'qwerty123',
-                                'database': 'test',
-                            }
+                    {
+                        'connections': {
+                            # Dict format for connection
+                            'default': {
+                                'engine': 'tortoise.backends.asyncpg',
+                                'credentials': {
+                                    'host': 'localhost',
+                                    'port': '5432',
+                                    'user': 'tortoise',
+                                    'password': 'qwerty123',
+                                    'database': 'test',
+                                }
+                            },
+                            # Using a DB_URL string
+                            'default': 'postgres://postgres:qwerty123@localhost:5432/events'
                         },
-                        # Using a DB_URL string
-                        'default': 'postgres://postgres:qwerty123@localhost:5432/events'
-                    },
-                    'apps': {
-                        'models': {
-                            'models': ['__main__'],
-                            # If no default_connection specified, defaults to 'default'
-                            'default_connection': 'default',
+                        'apps': {
+                            'models': {
+                                'models': ['__main__'],
+                                # If no default_connection specified, defaults to 'default'
+                                'default_connection': 'default',
+                            }
                         }
                     }
-                }
 
-        config_file:
+        :param config_file:
             Path to .json or .yml (if PyYAML installed) file containing config with
             same format as above.
-        db_url:
+        :param db_url:
             Use a DB_URL string. See :ref:`db_url`
-        modules:
+        :param modules:
             Dictionary of ``key``: [``list_of_modules``] that defined "apps" and modules that
             should be discovered for models.
-        _create_db:
+        :param _create_db:
             If ``True`` tries to create database for specified connections,
             could be used for testing purposes.
 
-        Raises
-        ------
-        ConfigurationError
-            For any configuration error
+        :raises ConfigurationError: For any configuration error
         """
         if cls._inited:
             await cls.close_connections()
@@ -730,10 +751,9 @@ class Tortoise:
         Will fail if schemas already exists, so it's not recommended to be used as part
         of application workflow
 
-        Parameters
-        ----------
-        safe:
-            When set to true, creates the table only when it does not already exist.
+        :param safe: When set to true, creates the table only when it does not already exist.
+
+        :raises ConfigurationError: When ``.init()`` has not been called.
         """
         if not cls._inited:
             raise ConfigurationError("You have to call .init() first before generating schemas")
@@ -745,6 +765,8 @@ class Tortoise:
         """
         Tries to drop all databases provided in config passed to ``.init()`` method.
         Normally should be used only for testing purposes.
+
+        :raises ConfigurationError: When ``.init()`` has not been called.
         """
         if not cls._inited:
             raise ConfigurationError("You have to call .init() first before deleting schemas")
@@ -781,4 +803,4 @@ def run_async(coro: Coroutine) -> None:
         loop.run_until_complete(Tortoise.close_connections())
 
 
-__version__ = "0.15.22"
+__version__ = "0.16.0"
