@@ -1,26 +1,21 @@
-from typing import List
+from typing import TYPE_CHECKING, List
 
-from tortoise import fields
 from tortoise.backends.base.schema_generator import BaseSchemaGenerator
 from tortoise.utils import get_escape_translation_table
 
+if TYPE_CHECKING:  # pragma: nocoverage
+    from tortoise.backends.asyncpg.client import AsyncpgDBClient
+
 
 class AsyncpgSchemaGenerator(BaseSchemaGenerator):
-    TABLE_COMMENT_TEMPLATE = "COMMENT ON TABLE {table} IS '{comment}';"
-    COLUMN_COMMNET_TEMPLATE = "COMMENT ON COLUMN {table}.{column} IS '{comment}';"
+    DIALECT = "postgres"
+    TABLE_COMMENT_TEMPLATE = "COMMENT ON TABLE \"{table}\" IS '{comment}';"
+    COLUMN_COMMNET_TEMPLATE = 'COMMENT ON COLUMN "{table}"."{column}" IS \'{comment}\';'
+    GENERATED_PK_TEMPLATE = '"{field_name}" {generated_sql}'
 
-    FIELD_TYPE_MAP = {
-        **BaseSchemaGenerator.FIELD_TYPE_MAP,
-        fields.JSONField: "JSONB",
-        fields.UUIDField: "UUID",
-    }
-
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
-        self.comments_array = []  # type: List[str]
-
-    def _get_primary_key_create_string(self, field_name: str, comment: str) -> str:
-        return '"{}" SERIAL NOT NULL PRIMARY KEY'.format(field_name)
+    def __init__(self, client: "AsyncpgDBClient") -> None:
+        super().__init__(client)
+        self.comments_array: List[str] = []
 
     def _escape_comment(self, comment: str) -> str:
         table = get_escape_translation_table()
