@@ -5,7 +5,7 @@ from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Tuple
 from pypika import Table
 from pypika.enums import SqlTypes
 from pypika.functions import Cast, Upper
-from pypika.terms import BasicCriterion, Criterion, Equality, Term, ValueWrapper
+from pypika.terms import Criterion, Term
 
 from tortoise.fields import Field
 from tortoise.fields.relational import BackwardFKRelation, ManyToManyFieldInstance
@@ -46,15 +46,11 @@ def string_encoder(value: Any, instance: "Model", field: Field) -> str:
 
 
 def is_in(field: Term, value: Any) -> Criterion:
-    if value:
-        return field.isin(value)
-    return BasicCriterion(Equality.eq, ValueWrapper(1), ValueWrapper(0))
+    return field.isin(value)
 
 
 def not_in(field: Term, value: Any) -> Criterion:
-    if value:
-        return field.notin(value) | field.isnull()
-    return BasicCriterion(Equality.eq, ValueWrapper(1), ValueWrapper(1))
+    return field.notin(value) | field.isnull()
 
 
 def between_and(field: Term, value: Tuple[Any, Any]) -> Criterion:
