@@ -1,16 +1,19 @@
-from typing import TYPE_CHECKING, Any, Optional, Type, cast
+from typing import TYPE_CHECKING, Any, Optional, Type, Union, cast
 
 from pypika import Table, functions
-from pypika.terms import AggregateFunction
+from pypika.functions import DistinctOptionFunction
+from pypika.terms import ArithmeticExpression
 from pypika.terms import Function as BaseFunction
 
 from tortoise.exceptions import ConfigurationError
-from tortoise.fields.relational import BackwardFKRelation, ForeignKeyFieldInstance, RelationalField
+from tortoise.expressions import F
+from tortoise.fields.relational import ForeignKeyFieldInstance, RelationalField
 
 if TYPE_CHECKING:  # pragma: nocoverage
     from tortoise.models import Model
     from tortoise.fields.base import Field
 
+
 ##############################################################################
 # Base
 ##############################################################################
@@ -40,11 +43,14 @@ class Function:
     # Enable populate_field_object where we want to try and preserve the field type.
     populate_field_object = False
 
-    def __init__(self, field: str, *default_values: Any) -> None:
+    def __init__(self, field: Union[str, F, ArithmeticExpression], *default_values: Any) -> None:
         self.field = field
         self.field_object: "Optional[Field]" = None
         self.default_values = default_values
 
+    def _get_function_field(self, field: str, *default_values):
+        return self.database_func(field, *default_values)
+
     def _resolve_field_for_model(
         self, model: "Type[Model]", table: Table, field: str, *default_values: Any
     ) -> dict:
@@ -54,22 +60,15 @@ class Function:
             if field_split[0] in model._meta.fetch_fields:
                 related_field = cast(RelationalField, model._meta.fields_map[field_split[0]])
                 related_field_meta = related_field.model_class._meta
-                related_table = related_field_meta.basetable
-                if isinstance(related_field, BackwardFKRelation):
-                    if table == related_table:
-                        related_table = related_table.as_(
-                            f"{table.get_table_name()}__{field_split[0]}"
-                        )
                 join = (table, field_split[0], related_field)
                 function_joins.append(join)
-                field = related_table[related_field_meta.db_pk_field]
+                field = related_field_meta.basetable[related_field_meta.db_pk_field]
             else:
                 field_object = model._meta.fields_map[field_split[0]]
                 if field_object.source_field:
                     field = table[field_object.source_field]
                 else:
                     field = table[field_split[0]]
-
                 if self.populate_field_object:
                     self.field_object = model._meta.fields_map.get(field_split[0], None)
                     if self.field_object:  # pragma: nobranch
@@ -79,7 +78,7 @@ class Function:
                         if func:
                             field = func(self.field_object, field)
 
-            function_field = self.database_func(field, *default_values)
+            function_field = self._get_function_field(field, *default_values)
             return {"joins": function_joins, "field": function_field}
 
         if field_split[0] not in model._meta.fetch_fields:
@@ -105,17 +104,40 @@ class Function:
             (to allow self referential joins)
         :return: Dict with keys ``"joins"`` and ``"fields"``
         """
-        function = self._resolve_field_for_model(model, table, self.field, *self.default_values)
-        function["joins"] = reversed(function["joins"])
-        return function
+
+        if isinstance(self.field, str):
+            function = self._resolve_field_for_model(model, table, self.field, *self.default_values)
+            function["joins"] = reversed(function["joins"])
+            return function
+        else:
+            field, field_object = F.resolver_arithmetic_expression(model, self.field)
+            if self.populate_field_object:
+                self.field_object = field_object
+            return {"joins": [], "field": self.database_func(field, *self.default_values)}
 
 
 class Aggregate(Function):
     """
     Base for SQL Aggregates.
+
+    :param field: Field name
+    :param default_values: Extra parameters to the function.
+    :param is_distinct: Flag for aggregate with distinction
     """
 
-    database_func = AggregateFunction
+    database_func = DistinctOptionFunction
+
+    def __init__(
+        self, field: Union[str, F, ArithmeticExpression], *default_values: Any, distinct=False
+    ) -> None:
+        super().__init__(field, *default_values)
+        self.distinct = distinct
+
+    def _get_function_field(self, field: str, *default_values):
+        if self.distinct:
+            return self.database_func(field, *default_values).distinct()
+        else:
+            return self.database_func(field, *default_values)
 
 
 ##############################################################################
@@ -135,7 +157,7 @@ class Trim(Function):
 
 class Length(Function):
     """
-    Returns lenth of text/blob.
+    Returns length of text/blob.
 
     :samp:`Length("{FIELD_NAME}")`
     """
