@@ -22,11 +22,17 @@ from typing import (
 from pypika import JoinType, Order, Table
 from pypika.functions import Count
 from pypika.queries import QueryBuilder
-from pypika.terms import ArithmeticExpression
+from pypika.terms import Term
 from typing_extensions import Protocol
 
 from tortoise.backends.base.client import BaseDBAsyncClient, Capabilities
-from tortoise.exceptions import DoesNotExist, FieldError, IntegrityError, MultipleObjectsReturned
+from tortoise.exceptions import (
+    DoesNotExist,
+    FieldError,
+    IntegrityError,
+    MultipleObjectsReturned,
+    ParamsError,
+)
 from tortoise.expressions import F
 from tortoise.fields.relational import (
     ForeignKeyFieldInstance,
@@ -57,7 +63,7 @@ class QuerySetSingle(Protocol[T_co]):
 
 
 class AwaitableQuery(Generic[MODEL]):
-    __slots__ = ("_joined_tables", "query", "model", "_db", "capabilities", "_annotations")
+    __slots__ = ("_joined_tables", "query", "model", "_db", "capabilities")
 
     def __init__(self, model: Type[MODEL]) -> None:
         self._joined_tables: List[Table] = []
@@ -65,7 +71,6 @@ class AwaitableQuery(Generic[MODEL]):
         self.query: QueryBuilder = QUERY
         self._db: BaseDBAsyncClient = None  # type: ignore
         self.capabilities: Capabilities = model._meta.db.capabilities
-        self._annotations: Dict[str, Function] = {}
 
     def resolve_filters(
         self,
@@ -82,8 +87,6 @@ class AwaitableQuery(Generic[MODEL]):
         :param annotations: Extra annotations to add.
         :param custom_filters:
         """
-        has_aggregate = self._resolve_annotate()
-
         modifier = QueryModifier()
         for node in q_objects:
             modifier &= node.resolve(model, annotations, custom_filters, model._meta.basetable)
@@ -97,11 +100,6 @@ class AwaitableQuery(Generic[MODEL]):
         self.query._wheres = where_criterion
         self.query._havings = having_criterion
 
-        if has_aggregate and (self._joined_tables or having_criterion):
-            self.query = self.query.groupby(
-                self.model._meta.basetable[self.model._meta.db_pk_field]
-            )
-
     def _join_table_by_field(
         self, table: Table, related_field_name: str, related_field: RelationalField
     ) -> Table:
@@ -138,6 +136,8 @@ class AwaitableQuery(Generic[MODEL]):
             (to allow self referential joins)
         :param orderings: What columns/order to order by
         :param annotations:  Annotations that may be ordered on
+
+        :raises FieldError: If a field provided does not exist in model.
         """
         # Do not apply default ordering for annotated queries to not mess them up
         if not orderings and self.model._meta.ordering and not annotations:
@@ -164,10 +164,9 @@ class AwaitableQuery(Generic[MODEL]):
                 annotation_info = annotation.resolve(self.model, table)
                 self.query = self.query.orderby(annotation_info["field"], order=ordering[1])
             else:
-                field_object = model._meta.fields_map.get(field_name)
-
+                field_object = self.model._meta.fields_map.get(field_name)
                 if not field_object:
-                    raise FieldError(f"Unknown field {field_name} for model {model.__name__}")
+                    raise FieldError(f"Unknown field {field_name} for model {self.model.__name__}")
                 field_name = field_object.source_field or field_name
                 field = getattr(table, field_name)
 
@@ -179,22 +178,6 @@ class AwaitableQuery(Generic[MODEL]):
 
                 self.query = self.query.orderby(field, order=ordering[1])
 
-    def _resolve_annotate(self) -> bool:
-        if not self._annotations:
-            return False
-
-        table = self.model._meta.basetable
-        annotation_info = {}
-        for key, annotation in self._annotations.items():
-            annotation_info[key] = annotation.resolve(self.model, table)
-
-        for key, info in annotation_info.items():
-            for join in info["joins"]:
-                self._join_table_by_field(*join)
-            self.query._select_other(info["field"].as_(key))
-
-        return any(info["field"].is_aggregate for info in annotation_info.values())
-
     def _make_query(self) -> None:
         raise NotImplementedError()  # pragma: nocoverage
 
@@ -208,8 +191,7 @@ class QuerySet(AwaitableQuery[MODEL]):
         "_prefetch_map",
         "_prefetch_queries",
         "_single",
-        "_get",
-        "_count",
+        "_raise_does_not_exist",
         "_db",
         "_limit",
         "_offset",
@@ -217,6 +199,7 @@ class QuerySet(AwaitableQuery[MODEL]):
         "_orderings",
         "_q_objects",
         "_distinct",
+        "_annotations",
         "_having",
         "_custom_filters",
     )
@@ -224,18 +207,17 @@ class QuerySet(AwaitableQuery[MODEL]):
     def __init__(self, model: Type[MODEL]) -> None:
         super().__init__(model)
         self.fields: Set[str] = model._meta.db_fields
-
         self._prefetch_map: Dict[str, Set[Union[str, Prefetch]]] = {}
         self._prefetch_queries: Dict[str, QuerySet] = {}
         self._single: bool = False
-        self._get: bool = False
-        self._count: bool = False
+        self._raise_does_not_exist: bool = False
         self._limit: Optional[int] = None
         self._offset: Optional[int] = None
         self._filter_kwargs: Dict[str, Any] = {}
         self._orderings: List[Tuple[str, Any]] = []
         self._q_objects: List[Q] = []
         self._distinct: bool = False
+        self._annotations: Dict[str, Function] = {}
         self._having: Dict[str, Any] = {}
         self._custom_filters: Dict[str, dict] = {}
 
@@ -248,8 +230,7 @@ class QuerySet(AwaitableQuery[MODEL]):
         queryset._prefetch_map = copy(self._prefetch_map)
         queryset._prefetch_queries = copy(self._prefetch_queries)
         queryset._single = self._single
-        queryset._get = self._get
-        queryset._count = self._count
+        queryset._raise_does_not_exist = self._raise_does_not_exist
         queryset._db = self._db
         queryset._limit = self._limit
         queryset._offset = self._offset
@@ -308,6 +289,8 @@ class QuerySet(AwaitableQuery[MODEL]):
             .order_by('name', '-tournament__name')
 
         Supports ordering by related models too.
+
+        :raises FieldError: If unknown field has been provided.
         """
         queryset = self._clone()
         new_ordering = []
@@ -326,7 +309,12 @@ class QuerySet(AwaitableQuery[MODEL]):
     def limit(self, limit: int) -> "QuerySet[MODEL]":
         """
         Limits QuerySet to given length.
+
+        :raises ParamsError: Limit should be non-negative number.
         """
+        if limit < 0:
+            raise ParamsError("Limit should be non-negative number")
+
         queryset = self._clone()
         queryset._limit = limit
         return queryset
@@ -334,7 +322,12 @@ class QuerySet(AwaitableQuery[MODEL]):
     def offset(self, offset: int) -> "QuerySet[MODEL]":
         """
         Query offset for QuerySet.
+
+        :raises ParamsError: Offset should be non-negative number.
         """
+        if offset < 0:
+            raise ParamsError("Offset should be non-negative number")
+
         queryset = self._clone()
         queryset._offset = offset
         if self.capabilities.requires_limit and queryset._limit is None:
@@ -355,6 +348,8 @@ class QuerySet(AwaitableQuery[MODEL]):
     def annotate(self, **kwargs: Function) -> "QuerySet[MODEL]":
         """
         Annotate result with aggregation or function result.
+
+        :raises TypeError: Value of kwarg is expected to be a ``Function`` instance.
         """
         queryset = self._clone()
         for key, annotation in kwargs.items():
@@ -385,8 +380,7 @@ class QuerySet(AwaitableQuery[MODEL]):
                 field
                 for field in self.model._meta.fields_map.keys()
                 if field in self.model._meta.db_fields
-            ]
-            + list(self._annotations.keys()),
+            ],
             distinct=self._distinct,
             limit=self._limit,
             offset=self._offset,
@@ -402,6 +396,8 @@ class QuerySet(AwaitableQuery[MODEL]):
         Can pass names of fields to fetch, or as a ``field_name='name_in_dict'`` kwarg.
 
         If no arguments are passed it will default to a dict containing all fields.
+
+        :raises FieldError: If duplicate key has been provided.
         """
         if args or kwargs:
             fields_for_select: Dict[str, str] = {}
@@ -415,13 +411,11 @@ class QuerySet(AwaitableQuery[MODEL]):
                     raise FieldError(f"Duplicate key {return_as}")
                 fields_for_select[return_as] = field
         else:
-            _fields = [
-                field
+            fields_for_select = {
+                field: field
                 for field in self.model._meta.fields_map.keys()
                 if field in self.model._meta.db_fields
-            ] + list(self._annotations.keys())
-
-            fields_for_select = {field: field for field in _fields}
+            }
 
         return ValuesQuery(
             db=self._db,
@@ -452,11 +446,11 @@ class QuerySet(AwaitableQuery[MODEL]):
         """
         Update all objects in QuerySet with given kwargs.
 
-        A usage example:
+        .. admonition: Example:
 
-        .. code-block:: py3
+            .. code-block:: py3
 
-            await Employee.filter(occupation='developer').update(salary=5000)
+                await Employee.filter(occupation='developer').update(salary=5000)
 
         Will instead of returning a resultset, update the data in the DB itself.
         """
@@ -505,7 +499,8 @@ class QuerySet(AwaitableQuery[MODEL]):
         """
         queryset = self.filter(*args, **kwargs)
         queryset._limit = 2
-        queryset._get = True
+        queryset._single = True
+        queryset._raise_does_not_exist = True
         return queryset  # type: ignore
 
     def get_or_none(self, *args: Q, **kwargs: Any) -> QuerySetSingle[Optional[MODEL]]:
@@ -520,6 +515,8 @@ class QuerySet(AwaitableQuery[MODEL]):
     def prefetch_related(self, *args: Union[str, Prefetch]) -> "QuerySet[MODEL]":
         """
         Like ``.fetch_related()`` on instance, but works on all objects in QuerySet.
+
+        :raises FieldError: If the field to prefetch on is not a relation, or not found.
         """
         queryset = self._clone()
         queryset._prefetch_map = {}
@@ -576,8 +573,26 @@ class QuerySet(AwaitableQuery[MODEL]):
         queryset._db = _db
         return queryset
 
+    def _resolve_annotate(self) -> None:
+        if not self._annotations:
+            return
+
+        table = self.model._meta.basetable
+        annotation_info = {}
+        for key, annotation in self._annotations.items():
+            annotation_info[key] = annotation.resolve(self.model, table)
+
+        if any(info["field"].is_aggregate for info in annotation_info.values()):
+            self.query = self.query.groupby(table[self.model._meta.db_pk_field])
+
+        for key, info in annotation_info.items():
+            for join in info["joins"]:
+                self._join_table_by_field(*join)
+            self.query._select_other(info["field"].as_(key))
+
     def _make_query(self) -> None:
         self.query = copy(self.model._meta.basequery_all_fields)
+        self._resolve_annotate()
         self.resolve_filters(
             model=self.model,
             q_objects=self._q_objects,
@@ -611,16 +626,12 @@ class QuerySet(AwaitableQuery[MODEL]):
             prefetch_map=self._prefetch_map,
             prefetch_queries=self._prefetch_queries,
         ).execute_select(self.query, custom_fields=list(self._annotations.keys()))
-        if self._get:
-            if len(instance_list) == 1:
-                return instance_list[0]
-            if not instance_list:
-                raise DoesNotExist("Object does not exist")
-            raise MultipleObjectsReturned("Multiple objects returned, expected exactly one")
         if self._single:
             if len(instance_list) == 1:
                 return instance_list[0]
             if not instance_list:
+                if self._raise_does_not_exist:
+                    raise DoesNotExist("Object does not exist")
                 return None  # type: ignore
             raise MultipleObjectsReturned("Multiple objects returned, expected exactly one")
         return instance_list
@@ -674,12 +685,12 @@ class UpdateQuery(AwaitableQuery):
                     db_field = self.model._meta.fields_db_projection[key]
                 except KeyError:
                     raise FieldError(f"Field {key} is virtual and can not be updated")
-                if not isinstance(value, (F, ArithmeticExpression)):
-                    value = executor.column_map[key](value, None)  # type: ignore
+                if isinstance(value, Term):
+                    value = F.resolver_arithmetic_expression(self.model, value)[0]
+                elif isinstance(value, Function):
+                    value = value.resolve(self.model, table)["field"]
                 else:
-                    value = F.resolver_arithmetic_expression(
-                        self.model._meta.fields_db_projection, value
-                    )
+                    value = executor.column_map[key](value, None)
 
             self.query = self.query.set(db_field, value)
 
@@ -826,7 +837,9 @@ class FieldSelectQuery(AwaitableQuery):
             )
 
         if field in self.annotations:
-            self._annotations[return_as] = self.annotations[field]
+            annotation = self.annotations[field]
+            annotation_info = annotation.resolve(self.model, table)
+            self.query._select_other(annotation_info["field"].as_(return_as))
             return
 
         field_split = field.split("__")
@@ -847,7 +860,7 @@ class FieldSelectQuery(AwaitableQuery):
             # return as is to get whole model objects
             return lambda x: x
 
-        if field in [x[1] for x in model._meta.db_native_fields]:
+        if field in (x[1] for x in model._meta.db_native_fields):
             return lambda x: x
 
         if field in self.annotations:
@@ -946,7 +959,7 @@ class ValuesListQuery(FieldSelectQuery):
         _, result = await self._db.execute_query(str(self.query))
         columns = [
             (key, self.resolve_to_python_value(self.model, name))
-            for key, name in sorted(list(self.fields.items()))
+            for key, name in sorted(self.fields.items())
         ]
         if self.flat:
             func = columns[0][1]
